#pragma once
#include <iostream>
#include <vector>
#include <iomanip>


struct Node {
    int ID;
    double x, y, z;
    bool flag;

    Node() = default;
    Node(int ID, double x, double y, double z, bool vertex) :
        ID(ID), x(x), y(y), z(z), flag(flag) {}

    bool operator == (const Node& A) const;

    bool operator < (const Node& A) const {
        return ID < A.ID;
    }

    friend std::ostream& operator << (std::ostream& out, const Node& A);
    friend std::ostream& operator << (std::ostream& out, const std::vector<Node>& A);
};

struct FiniteElement {
    int FE_ID;
    int material_ID;
    std::vector<int> nodes_ID;

    friend std::ostream& operator << (std::ostream& out, const FiniteElement& A);
    friend std::ostream& operator << (std::ostream& out, const std::vector<FiniteElement>& A);
};

struct BoundaryFiniteElement: public FiniteElement {
    int BFE_ID;
    int border_ID;

    friend std::ostream& operator << (std::ostream& out, const BoundaryFiniteElement& A);
    friend std::ostream& operator << (std::ostream& out, const std::vector<BoundaryFiniteElement>& A);
};

struct Edge {
    int first_node_ID;
    int last_node_ID;
    int center_ID;

    Edge() = default;
    Edge(int first_node_ID, int last_node_ID, int center_ID) :
        first_node_ID(first_node_ID), last_node_ID(last_node_ID), center_ID(center_ID) {}

    bool operator == (const Edge& A) const;

    bool operator < (const Edge& A) const {
        return center_ID < A.center_ID;
    }

    void UpdateCenter(int ID) {
        center_ID = ID;
    }
};